// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Automation;
using Microsoft.Azure.Management.Automation.Models;

namespace Microsoft.Azure.Management.Automation
{
    public static partial class DscNodeConfigurationOperationsExtensions
    {
        /// <summary>
        /// Retrieve the Dsc node configurations by node configuration.  (see
        /// http://aka.ms/azureautomationsdk/dscnodeconfigurations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscNodeConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='nodeConfigurationName'>
        /// Required. The Dsc node configuration name.
        /// </param>
        /// <returns>
        /// The response model for the get Dsc node configuration operation.
        /// </returns>
        public static DscNodeConfigurationGetResponse Get(this IDscNodeConfigurationOperations operations, string resourceGroupName, string automationAccount, string nodeConfigurationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscNodeConfigurationOperations)s).GetAsync(resourceGroupName, automationAccount, nodeConfigurationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve the Dsc node configurations by node configuration.  (see
        /// http://aka.ms/azureautomationsdk/dscnodeconfigurations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscNodeConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='nodeConfigurationName'>
        /// Required. The Dsc node configuration name.
        /// </param>
        /// <returns>
        /// The response model for the get Dsc node configuration operation.
        /// </returns>
        public static Task<DscNodeConfigurationGetResponse> GetAsync(this IDscNodeConfigurationOperations operations, string resourceGroupName, string automationAccount, string nodeConfigurationName)
        {
            return operations.GetAsync(resourceGroupName, automationAccount, nodeConfigurationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve a list of dsc node configurations.  (see
        /// http://aka.ms/azureautomationsdk/dscnodeconfigurations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscNodeConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// Optional. The parameters supplied to the list operation.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static DscNodeConfigurationListResponse List(this IDscNodeConfigurationOperations operations, string resourceGroupName, string automationAccount, DscNodeConfigurationListParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscNodeConfigurationOperations)s).ListAsync(resourceGroupName, automationAccount, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve a list of dsc node configurations.  (see
        /// http://aka.ms/azureautomationsdk/dscnodeconfigurations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscNodeConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// Optional. The parameters supplied to the list operation.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static Task<DscNodeConfigurationListResponse> ListAsync(this IDscNodeConfigurationOperations operations, string resourceGroupName, string automationAccount, DscNodeConfigurationListParameters parameters)
        {
            return operations.ListAsync(resourceGroupName, automationAccount, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve next list of dsc node configrations.  (see
        /// http://aka.ms/azureautomationsdk/dscnodeconfigurations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscNodeConfigurationOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The link to retrieve next set of items.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static DscNodeConfigurationListResponse ListNext(this IDscNodeConfigurationOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscNodeConfigurationOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve next list of dsc node configrations.  (see
        /// http://aka.ms/azureautomationsdk/dscnodeconfigurations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscNodeConfigurationOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The link to retrieve next set of items.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static Task<DscNodeConfigurationListResponse> ListNextAsync(this IDscNodeConfigurationOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
