// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Management.Sql;
using Microsoft.WindowsAzure.Management.Sql.Models;

namespace Microsoft.WindowsAzure.Management.Sql
{
    /// <summary>
    /// This is the main client class for interacting with the Azure SQL
    /// Database REST APIs.
    /// </summary>
    public static partial class DatabaseOperationOperationsExtensions
    {
        /// <summary>
        /// Returns information about a specific operation by using the
        /// operation Guid.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IDatabaseOperationOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server where the
        /// database is hosted.
        /// </param>
        /// <param name='operationGuid'>
        /// Required. The Guid of the Azure SQL Database operation to be
        /// obtained.
        /// </param>
        /// <returns>
        /// Represents the database operation for a given operation Guid.
        /// </returns>
        public static DatabaseOperationGetResponse Get(this IDatabaseOperationOperations operations, string serverName, string operationGuid)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDatabaseOperationOperations)s).GetAsync(serverName, operationGuid);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about a specific operation by using the
        /// operation Guid.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IDatabaseOperationOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server where the
        /// database is hosted.
        /// </param>
        /// <param name='operationGuid'>
        /// Required. The Guid of the Azure SQL Database operation to be
        /// obtained.
        /// </param>
        /// <returns>
        /// Represents the database operation for a given operation Guid.
        /// </returns>
        public static Task<DatabaseOperationGetResponse> GetAsync(this IDatabaseOperationOperations operations, string serverName, string operationGuid)
        {
            return operations.GetAsync(serverName, operationGuid, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieves all of the operations that took place on a specific
        /// database.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IDatabaseOperationOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server that hosts the
        /// database.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the database for which the operations should
        /// be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response containing the list of database operations
        /// for a given server or database.
        /// </returns>
        public static DatabaseOperationListResponse ListByDatabase(this IDatabaseOperationOperations operations, string serverName, string databaseName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDatabaseOperationOperations)s).ListByDatabaseAsync(serverName, databaseName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieves all of the operations that took place on a specific
        /// database.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IDatabaseOperationOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server that hosts the
        /// database.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the database for which the operations should
        /// be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response containing the list of database operations
        /// for a given server or database.
        /// </returns>
        public static Task<DatabaseOperationListResponse> ListByDatabaseAsync(this IDatabaseOperationOperations operations, string serverName, string databaseName)
        {
            return operations.ListByDatabaseAsync(serverName, databaseName, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieves all of the operations that occured on the Azure SQL
        /// Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IDatabaseOperationOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server to be queried.
        /// </param>
        /// <returns>
        /// Represents the response containing the list of database operations
        /// for a given server or database.
        /// </returns>
        public static DatabaseOperationListResponse ListByServer(this IDatabaseOperationOperations operations, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDatabaseOperationOperations)s).ListByServerAsync(serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieves all of the operations that occured on the Azure SQL
        /// Database Server.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Sql.IDatabaseOperationOperations.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server to be queried.
        /// </param>
        /// <returns>
        /// Represents the response containing the list of database operations
        /// for a given server or database.
        /// </returns>
        public static Task<DatabaseOperationListResponse> ListByServerAsync(this IDatabaseOperationOperations operations, string serverName)
        {
            return operations.ListByServerAsync(serverName, CancellationToken.None);
        }
    }
}
